# $Header$

#++ @file
# makefile zum Erstellen der Interface-Datenbasis.
#
# Aufruf: make --unix -f idb.mak -C <moduledir> MODCONTROL=<myModuleSpec>
#
# @author	Martin Weber
# @author	Frank Redeker
# @copyright	(c) Razorcat GmbH 2006
#
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#++
# Delete the default suffixes (since we use pattern rules)
#--
.SUFFIXES:

#++
# Command Definitions
#--
BASENAME := basename.exe
CP       := cp.exe -f
SED      := sed.exe
SHELL    := bash.exe
SORT     := sort.exe
CAT      := cat.exe
CUT      := cut.exe
HEAD     := head.exe
RM       := rm.exe
UNAME    := uname.exe
TOUCH    := touch.exe
CHMOD    := chmod.exe
PATHNAME := ts_pathname.exe
IDB      := idb32.exe
TKS      := tks.exe
TI       := ti.exe -exec
CPP      := cpp-3.4.4-mingw.exe

#++
# iso99
#--
ifdef ISO99
   ifeq (1,$(ISO99))
      IDB := idb32-iso99.exe
   endif
endif

#++
# Check modul control file
#--
ifndef MODCONTROL
   $(error Variable 'MODCONTROL' not specified.)
else
   include .idbbuild/$(MODCONTROL)
endif

#++
# Additional variables
#--
ifdef USE_ADDITIONAL_VARIABLES
   include .idbbuild/variables.makefile
endif

#++
# Check if source file list is present
#--
ifndef Mod_src_files
   $(error '$(MODCONTROL)' does not define variable 'Mod_src_files'.)
endif

#++
# Check if target specific control file is present
#--
ifndef Mod_tgt_control
   $(error .idbbuild/$(MODCONTROL) does not define variable 'Mod_tgt_control'.)
else
   # include target specific control file
   include $(Mod_tgt_control)
   ifndef Tgt_compiler
      $(error $(Mod_tgt_control) does not define variable 'Tgt_compiler'.)
   endif
   ifeq ($(findstring "iar-",$(Tgt_compiler)),"")
      $(error This makefile is only for use with IAR target compilers)
   endif
   ifeq ("$(Tgt_preproc)","")
      $(error No preprocessor call defined, please check if environment attributes `IDBMake Preprocessing` and `Preprocessor Call` are properly set)
   endif
   # older version of IAR ARM don't support --predef_macros
   ifneq ("$(Tgt_compiler)","iar-arm")
      ifneq ($(strip $(findstring $(Tgt_compiler),iar-78k-v4 iar-arm-v5 iar-m16c-v3 iar-msp430 iar-r32c iar-v850)),)
         predef_macros=
      endif
   endif
endif

#++
# include source file specific variables
#--
src_spec_files := $(addprefix .idbbuild/,$(addsuffix _spec.makefile,$(basename $(notdir $(Mod_src_files)))))
ifneq ($(strip $(src_spec_files)),)
   include $(src_spec_files)
endif

#++
# Dependencies to create the preprocessed files
#--
srcdeps := .idbbuild/srcdeps.makefile

#++
# All preprocessed source files
#--
ppfiles = $(addprefix .idbbuild/,$(addsuffix .i,$(notdir $(Mod_src_files))))

#++
# Dependencies for each preprocessed source
#--
depfiles = $(ppfiles:%.c.i=%.d)

#++
# Interface database
#--
idbfile := .database/.idb/module.midb.xml

#++
# Include dependecies
#--
-include $(srcdeps)

#++
# Include per source file dependencies
#--
ifneq ($(strip $(depfiles)),)
   -include $(depfiles)
endif

#++
#  Include filetimes makefile
#--
-include .idbbuild/filetimes.makefile

#++
# setup preprocessing
#--
CPP             = $(Tgt_preproc)
cppflags_target = -D__TESSY_IDBMAKE__ $(Tgt_cppflags) $(Tgt_includes:%=-I '%')

#++
# Preprocess a source file
#--
define PREPROCESS.c
   # create an empty macro definition file to avoid defines.exe to complain
   $(TOUCH) $@.def
   $(CPP) $(cppflags_target) $(Src_cppflags_$*) -DTESSY $(CPPFLAGS) $(Src_include_$*:%=-I'%') $(predef_macros) --preprocess=nl $@ $<
endef

#++
# Copy a preprocessed file and remove unrecognized keywords
#--
define IDBPREP.i
   $(TKS) -ignore-unknown $(Tgt_compiler)
endef

#++
# Compile a .d file (raw dependencies)
#--
define DEPEND.c
   # extract all included files unique
   $(SED) -n -e '/^# *[0-9][0-9]*.*/ s/^# *[0-9]*.*\(".*"\) *1.*/\1/gp' $< | $(SORT) -u >> $@.tmp
   # make short names
   $(PATHNAME) -s -r @$@.tmp >> $@.tmp.short
   # append to list of all dependency files
   for file in $$($(CAT) $@.tmp.short); \
   do \
      echo $$file >>.idbbuild/dependencies.all.d; \
   done
   # create dependency file
   echo ".idbbuild/filetimes.timestamp $< $@: \\" >$@
   $(CAT) $@.tmp.short >>$@
   echo ""  >>$@
   # remove temporary files
   $(RM) -f $@.tmp*
endef

#++
# Explicit rules
#--
.PHONY: all idb

#++
# Default target
#--
all: idb

filetimes : .idbbuild/filetimes.timestamp

#++
# Create interface database
#--
idb: $(idbfile)

#++
# Preprocess files for instrumentation
#--
preprocess: $(ppfiles)
	for file in $(ppfiles); \
	do \
		$(CP) $$file ts_p_$$($(BASENAME) $$file .i); \
	done

#++
# Dependency for preprocessed scource
#--
$(ppfiles): $(srcdeps)

#++
# Dependency for interface database
#--
$(idbfile): $(ppfiles) $(Mod_env_files) .idbbuild/environment.timestamp
	for file in $(ppfiles); \
	do \
		$(IDBPREP.i) $$file ts_p_$$($(BASENAME) $$file .i); \
	done
	if [ -f $(idbfile) ]; \
	then \
		$(TI) $(IDB) -new "$(Project)" "$(Module)" "$(Database)" ; echo $$? > .idbbuild/idbresult; \
	else \
		$(TI) $(IDB) "$(Project)" "$(Module)" "$(Database)" ; echo $$? > .idbbuild/idbresult; \
	fi

#++
# Create makefile
#--
.idbbuild/filetimes.makefile : $(depfiles)
	echo -n ".idbbuild/filetimes.timestamp: " >$@
	# add sourcefiles as dependency
	if [ ! -z "$(Mod_orginal_src_files)" ]; \
	then \
		for file in $(Mod_orginal_src_files); \
		do \
			echo " \\" >>$@; \
			echo -n $$file >>$@; \
		done \
	fi
	for file in $(Mod_src_files); \
	do \
		echo " \\" >>$@; \
		echo -n $$file >>$@; \
	done
	# add includefiles as dependency
	if [ -f .idbbuild/dependencies.all.d ]; \
	then \
		for file in $$($(SORT) -u .idbbuild/dependencies.all.d); \
		do \
			echo " \\" >>$@; \
			echo -n $$file >>$@; \
		done \
	fi
	for file in $(Mod_env_files); \
	do \
		echo " \\" >>$@; \
		echo -n $$file >>$@; \
	done
	echo -e "\n\techo \"52\" > .idbbuild/idbresult" >>$@
	echo -e "\techo \"idbmake: filetimes are in the future\"" >>$@
	echo -e "\techo $$^ > .idbbuild/dependency_files.txt" >>$@
	echo -e "\texit 2" >>$@
	# execute generated makefile to check if any of the
	@$(MAKE) -f $@

#++
# Implicit rules
#--

#++
# Implicit rule to generate a formatted dependency file from preprocessed file.
#--
.idbbuild/%.d: .idbbuild/%.c.i
	$(DEPEND.c)

#++
# Create a preprocessed file
#--
$(ppfiles) : .idbbuild/%.c.i:
	$(PREPROCESS.c)
